import 'dart:convert';
import 'package:http/http.dart';

extension ResponseExtension on Response {
  /// JSON decoded [body] of the request, as-is.
  get json => jsonDecode(body);

  /// Transforms this element to specified [Type] of element by applying the
  /// transformation function on the JSON decoded [body].
  T as<T>(T f(e)) => f(json);

  /// Casts the JSON decoded [body] as an iterable dynamic [List].
  List get asList => (json as List);

  /// Casts the JSON decoded [body] as an iterable [List] of specified [Type] by
  /// applying the transformation function to each element of this list.
  List<T> asListOf<T>(T f(e)) => asList.map(f).toList();
}
