import 'package:flutter_test/flutter_test.dart';

import 'package:asis/asis.dart';

import 'custom-object.dart';
import 'response-multiple.dart';
import 'response-single.dart';

void main() {
  group('[SINGLE]', () {
    group('[BASIC]', () {
      test('Decoded body', () {
        expect(
          singleResponse.json,
          singleEntry,
        );
      });

      test('Transformation of the decoded body as <T>', () {
        expect(
          singleResponse.as<Map>((e) => e['object']),
          singleEntry['object'],
        );
      });

      test('Transformation of the decoded body as <CustomObject>', () {
        expect(
          singleResponse
              .as<CustomObject>((e) => CustomObject.fromJson(e))
              .runtimeType,
          CustomObject,
        );
      });
    });

    group('[FUTURE]', () {
      test('Decoded body', () async {
        expect(
          await singleResponseFuture.json,
          singleEntry,
        );
      });

      test('Transformation of the decoded body as <T>', () async {
        expect(
          await singleResponseFuture.json.as<Map>((e) => e['object']),
          singleEntry['object'],
        );
      });

      test('Transformation of the decoded body as <CustomObject>', () async {
        expect(
          (await singleResponseFuture.json
                  .as<CustomObject>((e) => CustomObject.fromJson(e)))
              .runtimeType,
          CustomObject,
        );
      });
    });
  });

  group('[MULTIPLE]', () {
    group('[BASIC]', () {
      test('Decoded body', () {
        expect(
          multipleResponse.json,
          multipleEntries,
        );
      });

      test('Decoded body as a dynamic List with the helper asList', () {
        expect(
          multipleResponse.asList,
          multipleEntries,
        );
      });

      test('Decoded body as a List of T with the helper asList', () {
        expect(
          multipleResponse.asList.as<Map>((e) => e['object']),
          multipleEntries.map((e) => e['object']),
        );
      });

      test('Decoded body as a List of <CustomObject> with the helper asList',
          () {
        expect(
          multipleResponse.asList
              .as<CustomObject>(
                (e) => CustomObject.fromJson(e),
              )
              .runtimeType,
          equals(
            multipleEntries
                .map((e) => CustomObject.fromJson(e))
                .toList()
                .runtimeType,
          ),
        );
      });

      test('Decoded body as a List of T with the helper asListOf<T>', () {
        expect(
          multipleResponse.asListOf<Map>((e) => e['object']),
          multipleEntries.map((e) => e['object']),
        );
      });

      test(
          'Decoded body as a List of <CustomObject> with the helper asListOf<T>',
          () {
        expect(
          multipleResponse
              .asListOf<CustomObject>((e) => CustomObject.fromJson(e))
              .runtimeType,
          equals(
            multipleEntries
                .map((e) => CustomObject.fromJson(e))
                .toList()
                .runtimeType,
          ),
        );
      });

      test(
          'Transformation of the decoded body as a dynamic List without helper',
          () {
        expect(
          multipleResponse.as<List>((e) => e as List),
          multipleEntries,
        );
      });

      test('Transformation of the decoded body as a List of T without helper',
          () {
        expect(
          multipleResponse
              .as<List>((e) => e as List)
              .map<Map>((e) => e['object']),
          multipleEntries.map((e) => e['object']),
        );
      });

      test('Transformation on a transformed List<T> as another List<T>', () {
        expect(
          multipleResponse
              .as<List>((e) => e as List)
              .as<Map>((e) => e['object']),
          multipleEntries.map((e) => e['object']),
        );
      });
    });

    group('[FUTURE]', () {
      test('Decoded body', () async {
        expect(
          await multipleResponseFuture.json,
          multipleEntries,
        );
      });

      test('Decoded body as a dynamic List with the helper asList', () async {
        expect(
          await multipleResponseFuture.asList,
          multipleEntries,
        );
      });

      test('Decoded body as a List of T with the helper asList', () async {
        expect(
          await multipleResponseFuture.asList.as<Map>((e) => e['object']),
          multipleEntries.map((e) => e['object']),
        );
      });

      test('Decoded body as a List of T with the helper asListOf<T>', () async {
        expect(
          await multipleResponseFuture.asListOf<Map>((e) => e['object']),
          multipleEntries.map((e) => e['object']),
        );
      });

      test('Transformation of the decoded body as <T>', () async {
        expect(
          await multipleResponseFuture.as<Map>((e) => e[0]['object']),
          multipleEntries[0]['object'],
        );
      });

      test(
          'Transformation of the decoded body as a dynamic List without helper',
          () async {
        expect(
          await multipleResponseFuture.as<List>((e) => e as List),
          multipleEntries,
        );
      });

      test('Transformation of the decoded body as a List of T without helper',
          () async {
        expect(
          await multipleResponseFuture
              .as<List>((e) => e as List)
              .then((e) => e.map((e) => e['object'])),
          multipleEntries.map((e) => e['object']),
        );
      });

      test('Transformation on a transformed List<T> to another List<T>',
          () async {
        expect(
          await multipleResponseFuture
              .as<List>((e) => e as List)
              .as<Map>((e) => e['object']),
          multipleEntries.map((e) => e['object']),
        );
      });
    });
  });
}
